import 'dart:async';
import 'dart:convert';
import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_spinkit/flutter_spinkit.dart';
import 'package:http/http.dart' as http;
import './Constant/Constant.dart' as constant;import 'Constant/Constant.dart';
import 'blocs/PayUnitStream.dart';

class ApiService {


  initiatePayment({
    required transaction_id,
    required String transactionAmount,
    required String currency,
    required String transactionCallBackUrl,
    required context,
    required String X_API_KEY,
    required String productName,
    required String description,
    required merchandUserName,
    required merchandPassword,
    required mode,
    required Function actionAfterProccess,
  }) async {
    try {
      progressHub("Please wait", context);
      var data = jsonEncode({
        "currency": currency,
        "transaction_id": transaction_id,
        "total_amount": transactionAmount,
        "return_url": ""
      });
      var jsonResponse;
      var url = Uri.parse("${constant.baseUrl}/gateway/initialize");
      var response = await http.post(url,
          body: data,
          headers: getGlobalHeader(
              X_API_KEY, "$merchandUserName:$merchandPassword", mode));
      jsonResponse = json.decode(response.body);
      if (response.statusCode == 200 || response.statusCode == 201) {
        print("This is /initiatePayment response : $jsonResponse");
        if (jsonResponse != null) {
          List result = await getAllProviders(
            context: context,
            transactionId: jsonResponse['data']['t_id'],
            transactionAmount: jsonResponse['data']['t_sum'],
            transactionCallBackUrl: jsonResponse['data']['t_url'],
            merchandPassword: merchandPassword,
            merchandUserName: merchandUserName,
            X_API_KEY: X_API_KEY,
            mode: mode,
            actionAfterProccess: actionAfterProccess,
          );
          Navigator.of(context).pop();

          return {"result": result, "transaction_id": transaction_id};
        }
      } else {
        Navigator.of(context).pop();
        print("Response : $jsonResponse");
        if (jsonResponse['message'] == null) {
          constant.makeToast(jsonResponse['error'], context, Colors.red);
        } else {
          constant.makeToast(jsonResponse['message'], context, Colors.red);
        }
        actionAfterProccess(transaction_id, "FAILED");
        return false;
      }
    } catch (e) {
      print("Error : $e");
      Navigator.of(context).pop();
      showInfoDialog(
          context, "Something went wrong , please try again or later");
      actionAfterProccess(transaction_id, "FAILED");

      return false;
    }
  }

  getAllProviders({
    context,
    required String transactionId,
    required String transactionAmount,
    required String transactionCallBackUrl,
    required String merchandUserName,
    required String merchandPassword,
    required String X_API_KEY,
    required String mode,
    required Function actionAfterProccess,
  }) async {
    try {
      var jsonResponse;
      var url = Uri.parse(
          "${constant
              .baseUrl}/gateway/gateways?t_id=$transactionId&t_sum=$transactionAmount&t_url=$transactionCallBackUrl");
      var response = await http.get(url,
          headers: getGlobalHeader(
              X_API_KEY, "$merchandUserName:$merchandPassword", mode));
      jsonResponse = json.decode(response.body);
      if (response.statusCode == 200) {
        print("This is /getAllProviders response : $jsonResponse");
        if (jsonResponse != null) {
          return json.decode(response.body)['data'];
        }
      } else {
        if (jsonResponse != null) {
          print("Response :  : $jsonResponse");
          Navigator.of(context).pop();
          actionAfterProccess(transactionId, "FAILED");
          return [];
        }
      }
    } catch (e) {
      print("Error : $e");
      Navigator.of(context).pop();
      actionAfterProccess(transactionId, "FAILED");
      showInfoDialog(
          context, "Something went wrong , please try again or later");
    }
  }

  makePayment({
    required context,
    required String X_API_KEY,
    required String transactionAmount,
    required String transactionCallBackUrl,
    required String provider_short_tag,
    required String? phoneNumber,
    required Function actionAfterProccess,
    required String userName,
    required merchandUserName,
    required transaction_id,
    required merchandPassword,
    required mode,
    required currency,
    required paymentType,
  }) async {
    var response;
    var data = currency != null
        ? jsonEncode({
      'gateway': provider_short_tag,
      'amount': int.parse(transactionAmount),
      'transaction_id': transaction_id,
      'return_url': transactionCallBackUrl,
      'phone_number': phoneNumber,
      "name": userName,
      "currency": currency,
      "paymentType": paymentType
    })
        : jsonEncode({
      'gateway': provider_short_tag,
      'amount': int.parse(transactionAmount),
      'transaction_id': transaction_id,
      'return_url': transactionCallBackUrl,
      'phone_number': phoneNumber,
      "name": userName,
      "paymentType": paymentType
    });
    try {
      progressHub("Please wait", context);
      var jsonResponse;
      final BuildContext mycontext = context;

      if (provider_short_tag != "paypal") {
        // constant.closeAndToastPaymentAreMake(context);
      }
      var url = Uri.parse("${constant.baseUrl}/gateway/makepayment");

      response = await http.post(url,
          body: data,
          headers: getGlobalHeader(
              X_API_KEY, "$merchandUserName:$merchandPassword", mode));
      if (response.statusCode == 200 || response.statusCode == 201) {
        constant.makeToast(
            json.decode(response.body)['message'], context, Colors.green);
        jsonResponse = json.decode(response.body)["data"];

        if (jsonResponse["access_token"] != null) {
          var obj = {
            "order_id": jsonResponse["order_id"],
            "amount": jsonResponse["amount"],
            "currency": jsonResponse["currency"],
            "client_id": jsonResponse["client_id"],
            "accessToken": jsonResponse["access_token"],
          };
          getPaymentStatus(
              transactionId: transaction_id,
              paymentRef: jsonResponse["payment_ref"],
              payToken: jsonResponse["pay_token"],
              merchandUserName: merchandUserName,
              merchandPassword: merchandPassword,
              provider_short_tag: provider_short_tag,
              actionAfterProccess: actionAfterProccess,
              context: mycontext,
              mode: mode,
              X_API_KEY: X_API_KEY);
          return obj;
        } else {
          var obj = {
            "order_id": jsonResponse["order_id"],
            "amount": jsonResponse["amount"],
            "currency": jsonResponse["currency"],
            "client_id": jsonResponse["client_id"],
          };
          getPaymentStatus(
              transactionId: transaction_id,
              paymentRef: jsonResponse["payment_ref"],
              payToken: jsonResponse["pay_token"],
              merchandUserName: merchandUserName,
              merchandPassword: merchandPassword,
              provider_short_tag: provider_short_tag,
              actionAfterProccess: actionAfterProccess,
              context: mycontext,
              mode: mode,
              X_API_KEY: X_API_KEY);
          return obj;
        }
      } else {
        var responseData = json.decode(response.body);
        print(" Response : $responseData");
        Navigator.of(context).pop();
        actionAfterProccess(transaction_id, "FAILED");
        if (responseData != null) {
          payUnitStream.paymentSink.addError("Error");
          Navigator.of(context).pop();
          constant.makeToast(responseData['message'], context, Colors.red);
        }
      }
    } catch (e) {
      actionAfterProccess(transaction_id, "FAILED");
      Navigator.of(context).pop();
      showInfoDialog(
          context, "Something went wrong , please try again or later");
    }
  }

  getPaymentStatus({
    required context,
    required String transactionId,
    required String paymentRef,
    required String payToken,
    required String merchandUserName,
    required String merchandPassword,
    required String provider_short_tag,
    required Function actionAfterProccess,
    required String X_API_KEY,
    required String mode,
  }) async {
    try {
      print("This is getTransaction status");
      const time = const Duration(seconds: 4);
      Timer.periodic(time, (Timer timer) async {
        var jsonResponse;
        var url = Uri.parse(
            "${constant
                .baseUrl}/gateway/paymentstatus/$provider_short_tag/$transactionId?payment_ref=${paymentRef}&pay_token=${payToken}");
        var response = await http.get(url,
            headers: getGlobalHeader(
                X_API_KEY, "$merchandUserName:$merchandPassword", mode));
        print("This is response status/; ${response.statusCode}");
        jsonResponse = await jsonDecode(response.body);
        print("This is response status/; $jsonResponse");

        if (jsonResponse['status'] == "SUCCESSFUL") {
          print("Response SUCCESSFUL: $jsonResponse");
          Navigator.pop(context);
          actionAfterProccess(transactionId, "SUCCESS");
          timer.cancel();
        }

        if (jsonResponse['status'] == "FAILED") {
          Navigator.pop(context);
          print("Response FAILED: $jsonResponse");
          actionAfterProccess(transactionId, "FAILED");
          timer.cancel();
        }

      });
    } catch (e) {
      // Navigator.pop(context);
      actionAfterProccess(transactionId, "FAILED");
      showInfoDialog(
          context, "Something went wrong , please try again or later");
    }
  }

  String getEncode(String apiUserNameAndPassWord) {
    String credentials = "$apiUserNameAndPassWord";
    Codec<String, String> stringToBase64 = utf8.fuse(base64);
    String encoded = stringToBase64.encode(credentials);
    return encoded;
  }

  getGlobalHeader(String X_API_KEY, String apiUserNameAndPassWord,
      String mode) {
    return {
      'Authorization': 'Basic ${getEncode(apiUserNameAndPassWord)}',
      'Content-Type': 'application/json',
      'X-API-KEY': X_API_KEY,
      'mode': mode,
    };
  }

  progressHub(msg, context) {
    showDialog(
      barrierDismissible: false,
      barrierColor: Colors.black.withOpacity(0.5),
      context: context,
      builder: (context) =>
          WillPopScope(
            onWillPop: () async {
              return false;
            },
            child: Container(
              child: AlertDialog(
                shape:
                RoundedRectangleBorder(borderRadius: BorderRadius.circular(20)),
                backgroundColor: Colors.white.withOpacity(0.9),
                content: Container(
                  height: 200.0,
                  child: SpinKitSquareCircle(
                    color: payUnitGreenColor,
                    size: 50.0,
                  ),
                ),
              ),
            ),
          ),
    );
  }

  closeDialog(context) {
    // NativeProgressHud.hideWaiting();
    Navigator.of(context).pop();
    // Navigator.of(context).pop();
  }
}
